#define _COMMON_C_                                                                
                                                                               
/******************************************************************************
 *                             Include File Section                           *
 ******************************************************************************/
#include "common.h"                                                               
                                                                               
/******************************************************************************
 *                         Private Macro Define Section                       *
 ******************************************************************************/
                                                                               
/******************************************************************************
 *                             Struct Define Section                          *
 ******************************************************************************/
                                                                               
/******************************************************************************
 *                           Prototype Declare Section                        *
 ******************************************************************************/
static uint16 comDoRand(uint16 *val, uint16 taps);

/******************************************************************************
 *                        Global Variable Declare Section                     *
 ******************************************************************************/

/******************************************************************************
 *                     File Static Variable Declare Section                   *
 ******************************************************************************/
static uint16 u16Seed0 = 0xbeef;
static uint16 u16Seed1 = 0xface;

/******************************************************************************
 *                           Function Declare Section                         *
 ******************************************************************************/
/*============================== External Function ===========================*/

uint8 comCheckSum(const uint8 *pucBuf, uint8 ucLength, uint8 ucOffset)
{
    uint8  ucCheckSum = 0;

    pucBuf += ucOffset;

    while(ucLength--)
    {
        ucCheckSum += *pucBuf++;
    }

    return ucCheckSum; 
}

/*******************************************************************************
 * Function Name: CalCRC
 * Decription   : ַCRCУ
 * Calls        : 
 * Called By    : 
 * Input        : ptr: ַָָ
                  len: ַ  
 * Output       : None
 * Return Value : 16bitļ
 * Others       : None
 *------------------------------ Revision History ------------------------------
 * No. Version Date       Revised by   Item       Description     
 * 1   V1.0    2011-03-08 Lihao          Original Version          
 ******************************************************************************/
#define CRC_POLY 0x8408
uint16 comCalCRC16(uint8 *pucBuf, uint16 uwLength)
{
    uint16 uiCRCValue=0xFFFF;
    uint8  ucLoop;
    while(uwLength--)
    {
        uiCRCValue ^= *pucBuf++;
        for(ucLoop=0; ucLoop<8; ucLoop++)
        {
            if(uiCRCValue & 0x0001)
            {
                uiCRCValue >>= 1;
                uiCRCValue ^= CRC_POLY; 
            }
            else 
            {
                uiCRCValue >>= 1;
            }
        }
    }
    uiCRCValue ^= 0xFFFF;
    return uiCRCValue;
} 

/*******************************************************************************
 * Function Name: comMEMCPY
 * Decription   : srcָڴcountԪصdestָڴ
 *                srcdestָڴص
 *                ָdestsrcָԪͿunsigned charunsigned long͡
 * Calls        : 
 * Called By    : 
 * Input        : 
 * Output       : None
 * Return Value : ָdestָ롣
 * Others       : None
 *------------------------------ Revision History ------------------------------
 * No. Version Date       Revised by   Item       Description     
 *          
 ******************************************************************************/
void *comMEMCPY(void *dest, const void *src, unsigned int n)
{
    int longs, bytes;
    unsigned long *dpl = (unsigned long *)dest;
    unsigned long *spl = (unsigned long *)src;
    unsigned char  *dpb, *spb;

    if ((dest != NULL) && (src != NULL) && (n > 0))
    {
        bytes = (n & 0x3);
        longs = (n - bytes) >> 2;
    
        while (longs--)
            *dpl++ = *spl++;
        
        dpb = (unsigned char *)dpl;
        spb = (unsigned char *)spl;
        
        while (bytes--)
            *dpb++ = *spb++;
    }
    return dest;
}

/*******************************************************************************
 * Function Name: comMEMSET
 * Decription   : sָĳһڴеÿֽڵȫΪchָASCIIֵ,
 *                ĴСɵָ,ͨΪڴʼ,
 *                ䷵ֵΪָSָ롣
 * Calls        : 
 * Called By    : 
 * Input        :
 * Output       : None
 * Return Value : ֵΪָSָ
 * Others       : None
 *------------------------------ Revision History ------------------------------
 * No. Version Date       Revised by   Item       Description     
 *          
 ******************************************************************************/
void *comMEMSET(void *pbuf, int value, unsigned int n)
{
    /* Not optimized, but very portable */
    unsigned char *sp = (unsigned char *)pbuf;

    if ((pbuf != NULL) && (n > 0))
    {
        while (n--)
        {
            *sp++ = (unsigned char)value;
        }
    }
    return pbuf;
}

/*******************************************************************************
 * Function Name: comMEMCMP
 * Decription   : Ƚڴbuf1buf2ǰnֽ
 * Calls        : 
 * Called By    : 
 * Input        :
 * Output       : None
 * Return Value : buf1<buf2ʱֵ<0
 *                buf1=buf2ʱֵ=0
 *                buf1>buf2ʱֵ>0
 * Others       : None
 *------------------------------ Revision History ------------------------------
 * No. Version Date       Revised by   Item       Description     
 *          
 ******************************************************************************/
int comMEMCMP(const void *buf1, const void *buf2, unsigned int n) 
{
    const unsigned char *cs1 = buf1, *cs2 = buf2;

    for (; n-- > 0; cs1++, cs2++)
    {
        if (*cs1 != *cs2)
        {
            return *cs1 - *cs2;
        }
    }
    return 0;
}


/*******************************************************************************
 * Function Name: comSrand
 * Decription   : ӣֻڳտʼʱ(comRand()֮ǰ)һΡ
 * Calls        : 
 * Called By    : 
 * Input        : None
 * Output       : None
 * Return Value : None
 * Others       : None
 *------------------------------ Revision History ------------------------------
 * No. Version Date       Revised by   Item       Description     
 * 1   V1.0    2011-03-08 Lihao          Original Version          
 ******************************************************************************/
void comSrand(uint32 u32Seed)
{
    u16Seed0 = (uint16)u32Seed;
    if(u16Seed0 == 0)
    {
        u16Seed0 = 0xbeef;
    }

    u16Seed1 = (uint16)(u32Seed >> 16);
    if(u16Seed1 == 0)
    {
        u16Seed1 = 0xface;
    }
}

/*******************************************************************************
 * Function Name: comRand
 * Decription   : һ1--uwMax֮
 * Calls        : 
 * Called By    : 
 * Input        : None
 * Output       : None
 * Return Value : None
 * Others       : None
 *------------------------------ Revision History ------------------------------
 * No. Version Date       Revised by   Item       Description     
 * 1   V1.0    2011-03-08 Lihao          Original Version          
 ******************************************************************************/
uint16 comRand(uint16 uwMax)
{
   uint16 val;

   val = comDoRand(&u16Seed0, 0x0062) ^ comDoRand(&u16Seed1, 0x100B);
   val = (uwMax == 0) ? 1 : ((val%uwMax) + 1);
   return val;
}

/*============================== Internal Function ===========================*/
/*******************************************************************************
 * Function Name: comDoRand
 * Decription   :  ʵ塣
 * Calls        : 
 * Called By    : 
 * Input        : None
 * Output       : None
 * Return Value : None
 * Others       : None
 *------------------------------ Revision History ------------------------------
 * No. Version Date       Revised by   Item       Description     
 * 1   V1.0    2011-03-08 Lihao          Original Version          
 ******************************************************************************/
static uint16 comDoRand(uint16 *val, uint16 taps)
{
    uint16 newVal = *val;

    if(newVal & 0x8000)
    {
        newVal ^= taps;
    }
    *val = newVal << 1;
    return newVal;
}

