                                                                             
/******************************************************************************
 *                             Multi-Include-Prevent                          *
 ******************************************************************************/
#ifndef _SPI_H_                                                                
#define _SPI_H_                                                                

#include "common.h"
#include "hal.h"
#include <io_r8cl35c.h>


/******************************************************************************
 *                              Debug Switch Section                          *
 ******************************************************************************/
                                                                               
/******************************************************************************
 *                             Include File Section                           *
 ******************************************************************************/

/******************************************************************************
 *                          Pubilc Macro Define Section                       *
 ******************************************************************************/
#ifdef _SPI_C_
#define SPI_EXT
#else
#define SPI_EXT extern
#endif
                                                                        
/******************************************************************************
 *                        Global Variable Declare Section                     *
 ******************************************************************************/
                                                                               
/******************************************************************************
 *                            Function Declare Section                        *
 ******************************************************************************/
typedef struct 
{
   uint8                  :1;
   uint8                  :1;
   uint8                  :1;
   uint8  IicSsuEn        :1;
   uint8  RDEn            :1;
   uint8  RCEn            :1;
   uint8  RGEn            :1;
   uint8                  :1;
}st_Mstcr;

typedef union 
{
 uint8     MstcrReg;
 st_Mstcr  stmstcr;
}unSsu_MstcrReg; 
 

typedef  struct 
{
   uint8  Iic1_Ssu0_Sel      :1;
   uint8                     :1;
   uint8                     :1;
   uint8                     :1;
   uint8                     :1;
   uint8                     :1;
   uint8                     :1;
   uint8                     :1;
}st_ssuiicsr;
 
typedef union 
{
 uint8 SsuiicsrReg;
 st_ssuiicsr stssuiicsr;
}unSsu_SsuiicsrReg;


typedef struct 
{
   uint8  Send_Bits           :4;
   uint8                     :4;
}st_ssbr;

typedef union 
{
 uint8 Ssbr_Reg;
 st_ssbr stssbr;
}unSsu_Ssbr_Reg; 

#define SPI_SEND_16BITS   0
#define SPI_SEND_8BITS    8
#define SPI_SEND_9BITS    9
#define SPI_SEND_10BITS   10
#define SPI_SEND_11BITS   11
#define SPI_SEND_12BITS   12
#define SPI_SEND_13BITS   13
#define SPI_SEND_14BITS   14
#define SPI_SEND_15BITS   15

typedef struct 
{
   uint8   CLK_SEL            :3;       
   uint8                     :1;
   uint8                     :1;
   uint8   M1_OR_S0           :1;        //ѡ
   uint8   RSTOP1             :1;        //һֽںǷ
   uint8                     :1;
}st_sscrh;

typedef union 
{
 uint8 Sscrh_Reg;
 st_sscrh stsscrh;
} unSsu_Sscrh_Reg;
#define  SPI_CLK_F1_256       0
#define  SPI_CLK_F1_128       1
#define  SPI_CLK_F1_64        2
#define  SPI_CLK_F1_32        3 
#define  SPI_CLK_F1_16        4
#define  SPI_CLK_F1_8         5
#define  SPI_CLK_F1_4         6

typedef struct 
{
   uint8                     :5;
   uint8  Even1_Odd0_Change  :1;   //ѡ仯ڻػż
   uint8  High0_Low1_Stop    :1;    //ѡʱֹͣʱΪߵƽ͵ƽ
   uint8  MSB0_LSB1_First    :1;
}st_ssmr;
  
typedef union 
{
 uint8 Ssmr_Reg;
 st_ssmr stssmr;
}unSsu_Ssmr_Reg;

 #define  SPI_TX_ENABLE             sser|=0x10
 #define  SPI_TX_DISABLE            sser&=~0x10
 #define  SPI_RX_ENABLE             sser|= 0x08
 #define  SPI_RX_DISABLE            sser&=~0x08
 #define  SPI_TX_BUF_INT_ENABLE     sser|=0X80
 #define  SPI_TX_BUF_INT_DISABLE    sser&=~0X80
 #define  SPI_TX_END_INT_ENABLE     sser|=0X40
 #define  SPI_TX_END_INT_DISABLE    sser&=~0X40
 #define  SPI_RX_END_INT_ENABLE     sser|=0X20
 #define  SPI_RX_END_INT_DISABLE    sser&=~0X20

typedef struct 
{
   uint8   FourWire1_Sel        :1;
   uint8   SCS_CMOS_N1          :1;
   uint8   SO_CMOS_N1           :1;
   uint8   SCK_CMOS_N1          :1;
   uint8   CS_Config            :2;
   uint8   SCK_PORT0_CLOCK1     :1;
   uint8   BIDE_TWO1_ONE        :1;//ѡ˫ߵ
}st_ssmr2;

typedef union 
{
 uint8 Ssmr2_Reg;
 st_ssmr2 stssmr2;
}unSsu_Ssmr2_Reg;

#define SPI_CS_PORT      0
#define SPI_CS_INPUT     1
#define SPI_CS_OUTPUT    2


#define  SPI_IS_RX_END          (sssr&0X80)
#define  SPI_IS_TX_END          (sssr&0X40)
#define  SPI_IS_TX_BUF_EMPTY    (sssr&0X80)
#define  SPI_IS_CE              (sssr&0X01)
#define  SPI_IS_ORER            (sssr&0X04)

#define  SPI_CLR_RX_END           SSSR&=~0X80 
#define  SPI_CLR_TX_END           sssr&=~0X40
#define  SPI_CLR_TX_BUF_EMPTY     sssr&=~0X80
#define  SPI_CLR_CE               sssr&=~0X01
#define  SPI_CLR_ORER             sssr&=~0X04

#define  SPI_RX_ONE_STOP       rsstp_sscrh =1   
#define  SPI_RX_ONE_GOON       rsstp_sscrh =0
 

#define SPI_TX_BUF      sstdr
#define SPI_RX_BUF      ssrdr

SPI_EXT void halSpiInit(void);
SPI_EXT void spiWriteAddrByte(uint8 address,uint8 data);
SPI_EXT uint8 spiReadAddrByte(uint8 address);

#endif
